#include <stdlib.h>
#include "clist.h"

t_List* ListCreate ()
{
	t_List*	l_p_NewList;

	l_p_NewList = (t_List*) malloc (sizeof (t_List));
	if (l_p_NewList == 0)
	{
		return (0);
	}
	l_p_NewList->p_Current	= 0;
	l_p_NewList->p_Top	= 0;
	l_p_NewList->p_Bottom	= 0;
	return (l_p_NewList);
}

/*
 * return:	0 - first element does not exists
 * 		1 - first element exists
 */
int ListSetStart ( t_List* p_List )
{
	if (p_List != 0)
	{
		p_List->p_Current = p_List->p_Top;
		return (1);
	}
	return (0);
}

/*
 * return:	0 - last element does not exists
 * 		1 - last element exists
 */
int ListSetEnd ( t_List* p_List )
{
	if (p_List != 0)
	{
		p_List->p_Current = p_List->p_Bottom;
		return (1);
	}
	return (0);
}

/*
 * return:	0 - current is 0
 * 		1 - current exists (!NULL)
 */
int ListCurrent ( t_List* p_List )
{
	if (p_List->p_Current)
		return (1);
	return (0);
}

/*
 * return p_List->current and set it to the next element
 * return:	0  - there is no element
 * 		!0 - the element
 */
void* ListNext ( t_List* p_List )
{
	void*	vp_Content;

	if ((p_List != 0) && (p_List->p_Current != 0))
	{
		vp_Content = p_List->p_Current->vp_Content;
		p_List->p_Current = p_List->p_Current->p_Next;
		return (vp_Content);
	}
	return (0);
}

/*
 * return:	0 - there is no prev element
 * 		1 - there is a prev element
 */
void* ListPrev ( t_List* p_List )
{
	void*	vp_Content;

	if ((p_List != 0) && (p_List->p_Current != 0))
	{
		vp_Content = p_List->p_Current->vp_Content;
		p_List->p_Current = p_List->p_Current->p_Prev;
		return (vp_Content);
	}
	return (0);
}

int ListPushFront ( t_List* p_List, void* vp_Content )
{
	t_ListNode*	l_p_NewNode;

	if (p_List == 0)
	{
		return (-1);
	}
	l_p_NewNode = (t_ListNode*) malloc (sizeof (t_ListNode));
	if (l_p_NewNode == 0)
	{
		return (-1);
	}
	l_p_NewNode->vp_Content	= vp_Content;
	l_p_NewNode->p_Prev	= 0;
	l_p_NewNode->p_Next	= p_List->p_Top;
	if (p_List->p_Top != 0)
	{
		p_List->p_Top->p_Prev	= l_p_NewNode;
	}
	if (p_List->p_Bottom == 0)
	{
		p_List->p_Bottom = l_p_NewNode;
	}
	p_List->p_Top		= l_p_NewNode;
	p_List->p_Current	= p_List->p_Top;
	return (0);
}

int ListPushBack ( t_List* p_List, void* vp_Content )
{
	t_ListNode*	l_p_NewNode;

	if (p_List == 0)
	{
		return (-1);
	}
	l_p_NewNode = (t_ListNode*) malloc (sizeof (t_ListNode));
	if (l_p_NewNode == 0)
	{
		return (-1);
	}
	l_p_NewNode->vp_Content	= vp_Content;
	l_p_NewNode->p_Prev	= p_List->p_Bottom;
	l_p_NewNode->p_Next	= 0;
	if (p_List->p_Bottom != 0)
	{
		p_List->p_Bottom->p_Next= l_p_NewNode;
	}
	if (p_List->p_Top == 0)
	{
		p_List->p_Top = l_p_NewNode;
	}
	p_List->p_Bottom	= l_p_NewNode;
	p_List->p_Current	= p_List->p_Bottom;
	return (0);
}

void* ListPopFront ( t_List* p_List )
{
	t_ListNode*	l_p_Node;
	void*		l_vp_Content;

	if ((p_List == 0) || (p_List->p_Top == 0))
	{
		return (0);
	}
	l_p_Node = p_List->p_Top;
	p_List->p_Top = p_List->p_Top->p_Next;
	if (p_List->p_Bottom == l_p_Node)
		p_List->p_Bottom = p_List->p_Top;
	p_List->p_Current = p_List->p_Top;
	if (l_p_Node->p_Prev != 0)
	{
		l_p_Node->p_Prev->p_Next = l_p_Node->p_Next;
	}
	if (l_p_Node->p_Next != 0)
	{
		l_p_Node->p_Next->p_Prev = l_p_Node->p_Prev;
	}
	l_vp_Content = l_p_Node->vp_Content;
	free (l_p_Node);
	l_p_Node = 0;
	return (l_vp_Content);
}

void* ListPopBack ( t_List* p_List )
{
	t_ListNode*	l_p_Node;
	void*		l_vp_Content;

	if ((p_List == 0) || (p_List->p_Bottom == 0))
	{
		return(0);
	}
	l_p_Node = p_List->p_Bottom;
	p_List->p_Bottom = p_List->p_Bottom->p_Prev;
	if (p_List->p_Top == l_p_Node)
		p_List->p_Top = p_List->p_Bottom;
	p_List->p_Current = p_List->p_Bottom;
	if (l_p_Node->p_Prev != 0)
	{
		l_p_Node->p_Prev->p_Next = l_p_Node->p_Next;
	}
	if (l_p_Node->p_Next != 0)
	{
		l_p_Node->p_Next->p_Prev = l_p_Node->p_Prev;
	}
	l_vp_Content = l_p_Node->vp_Content;
	free (l_p_Node);
	l_p_Node = 0;
	return (l_vp_Content);
}

void* ListTop ( t_List* p_List )
{
	if ((p_List != 0) && (p_List->p_Top != 0))
	{
		return (p_List->p_Top->vp_Content);
	}
	return (0);
}

void* ListBottom ( t_List* p_List )
{
	if ((p_List != 0) && (p_List->p_Bottom != 0))
	{
		return (p_List->p_Bottom->vp_Content);
	}
	return (0);
}

int ListEmpty ( t_List* p_List )
{
	if ((p_List == 0) || (p_List->p_Top == 0))
	{
		return (1);
	}
	return (0);
}

void ListDestroy ( t_List* p_List )
{
	t_ListNode*	l_p_Node;

	if (p_List == 0)
	{
		return;
	}
	while (p_List->p_Top != 0)
	{
		l_p_Node = p_List->p_Top;
		p_List->p_Top = p_List->p_Top->p_Next;
		free (l_p_Node);
		l_p_Node = 0;
	};
	free (p_List);
	p_List = 0;
}

